
local localhost = {}

local async = require("copas.async")
local util = require("sga.util")

function localhost.read_file(self, filename)
   return util.read_file(filename)
end

function localhost.write_file(self, filename, data)
   return util.write_file(filename, data)
end

function localhost.remove(self, filename)
   return os.remove(filename)
end

local function in_main_thread()
   local curr, is_main = coroutine.running()
   return curr == nil or is_main -- Lua 5.1 and 5.2+
end

function localhost.run(self, cmdline)
   local err_file = os.tmpname()
   local popen = in_main_thread and io.popen or async.io_popen
   local fd = popen(cmdline.." 2> "..err_file, "r")
   local stdout = fd:read("*a")
   fd:close()
   local stderr = util.read_file(err_file)
   os.remove(err_file)
   return stdout, stderr
end

function localhost.run_write(self, cmdline, data)
   local popen = in_main_thread and io.popen or async.io_popen
   local fd = popen(cmdline, "w")
   fd:write(data)
   return fd:close()
end

function localhost.run_ok(self, cmdline)
   local execute = in_main_thread and io.popen or async.io_popen
   local ok = execute(cmdline)
   return ok == true or ok == 0
end

function localhost.time(self)
   return os.time()
end

function localhost.new()
   local self = {}
   setmetatable(self, { __index = localhost } )
   return self
end

return localhost
